@@ -34,8 +34,7 @@
 #include "brw_vs.h"
 #include "brw_util.h"
 #include "brw_state.h"
-#include "program.h"
-#include "shader/arbprogparse.h"
+#include "shader/prog_print.h"
 
 
 
@@ -54,12 +53,11 @@ static void do_vs_prog( struct brw_context *brw,
    c.vp = vp;
 
    c.prog_data.outputs_written = vp->program.Base.OutputsWritten;
-   c.prog_data.inputs_read = brw_translate_inputs(brw->intel.ctx.VertexProgram._Enabled,
-						  vp->program.Base.InputsRead);
+   c.prog_data.inputs_read = vp->program.Base.InputsRead;
 
    if (c.key.copy_edgeflag) {
       c.prog_data.outputs_written |= 1<<VERT_RESULT_EDGE;
-      c.prog_data.inputs_read |= 1<<BRW_ATTRIB_EDGEFLAG;
+      c.prog_data.inputs_read |= 1<<VERT_ATTRIB_EDGEFLAG;
    }
 
    if (0)
@@ -105,6 +103,11 @@ static void brw_upload_vs_prog( struct brw_context *brw )
    key.copy_edgeflag = (brw->attribs.Polygon->FrontMode != GL_FILL ||
 			brw->attribs.Polygon->BackMode != GL_FILL);
 
+   /* BRW_NEW_METAOPS
+    */
+   if (brw->metaops.active)
+      key.know_w_is_one = 1;
+
    /* Make an early check for the key.
     */
    if (brw_search_cache(&brw->cache[BRW_VS_PROG], 
@@ -122,7 +125,7 @@ static void brw_upload_vs_prog( struct brw_context *brw )
 const struct brw_tracked_state brw_vs_prog = {
    .dirty = {
       .mesa  = _NEW_TRANSFORM | _NEW_POLYGON,
-      .brw   = BRW_NEW_VERTEX_PROGRAM,
+      .brw   = BRW_NEW_VERTEX_PROGRAM | BRW_NEW_METAOPS,
       .cache = 0
    },
    .update = brw_upload_vs_prog
